/*
 *  Copyright (C) 2002-2004 Aleksey Sugonyaev & 3OS.Team ( http://www.3os.ru/ )
 *  Licensed under the terms and conditions of the GNU Lesser General Public License
 */
/*
     CCpuInfo class implementation
*/

#include "cpuinfo.hpp"
#include "heap.hpp"


//     cpuid  
#define cpuid   _asm _emit 0x0F _asm _emit 0xA2


/*
     
*/
CCpuInfo::CCpuInfo(void)
{
    _StdFeatures = 0;
    _ExtIntelFeatures = 0;
    
    /*
          -   cpuid ?

          ,    cpuid 
         :     21   EFLAGS.
          -  cpuid  .

         cpuid     Intel 
         i486    AMD   Am486DX4
    */
    register dword old_flags, new_flags;

    _asm pushfd                        //   EFLAGS 
    _asm pop old_flags                 // old_flags
    new_flags = old_flags ^ 0x00200000;//  21- 
    _asm push new_flags                //   
    _asm popfd                         //    EFLAGS.
    _asm pushfd                        //   EFLAGS 
    _asm pop new_flags                 // new_flags

    _CpuId_Supported = new_flags != old_flags;

    if (!_CpuId_Supported)
        return;

    //  

    char *pszVendorId = _VendorId;
    CPU_VENDOR CpuVendor;
    dword dwBasicMaxValue;

    _asm
    {
        xor eax, eax
        cpuid

        mov dwBasicMaxValue, eax

        mov esi, pszVendorId
        mov [esi], ebx
        mov [esi + 4], edx
        mov [esi + 8], ecx
        mov [esi + 12], 0
        
        mov CpuVendor, ebx
    }
    _CpuVendor = CpuVendor;
    
    if (dwBasicMaxValue >= 1)
    {
        dword dwStdFeatures;
        dword dwExtIntelFeatures;

        _asm
        {
            mov eax, 1
            cpuid

            mov dwStdFeatures, edx
            mov dwExtIntelFeatures, ecx
        }
        
        _StdFeatures = dwStdFeatures;
        _ExtIntelFeatures = dwExtIntelFeatures;
    }

    dword dwExtAMDFeatures;
    
    _asm
    {
        mov eax, 0x80000000
        cpuid

        cmp eax, 0x80000001
        xor edx, edx
        jl quit

        inc eax
        cpuid
    quit:
        mov dwExtAMDFeatures, edx
    }

    _ExtAMDFeatures = dwExtAMDFeatures;
}


const char* CCpuInfo::GetVendorId(void) {return _VendorId;}

const dword CCpuInfo::GetStdFeatures(void) {return _StdFeatures;}

const dword CCpuInfo::GetExtIntelFeatures(void) {return _ExtIntelFeatures;}

const dword CCpuInfo::GetExtAMDFeatures(void) {return _ExtAMDFeatures;}
