/*
 *  Copyright (C) 2002-2004 Aleksey Sugonyaev & 3OS.Team ( http://www.3os.ru/ )
 *  Licensed under the terms and conditions of the GNU Lesser General Public License
 */
/*
    CCpuInfo class definition
*/


#ifndef __CPUINFO

#define __CPUINFO

#include "ctype.hpp"    //   3OS (byte & dword)

/*
       CPU
    (     AMD   Intel).

          Intel.
      AMD      .

        (  .
       ).
*/
#define SFF_FPU         0x00000001      // Floating-point unit on-chip
#define SFF_VME         0x00000002      // Virtual Mode Extension
#define SFF_DE          0x00000004      // Debugging Extension
#define SFF_PSE         0x00000008      // Page Size Extension
#define SFF_TSC         0x00000010      // Time Stamp Counter
#define SFF_MSR         0x00000020      // Model Specific Registers
#define SFF_PAE         0x00000040      // Physical Address Extension
#define SFF_MCE         0x00000080      // Machine Check Exception
#define SFF_CX8         0x00000100      // CMPXCHG8 Instruction Supported
#define SFF_APIC        0x00000200      // On-chip APIC Hardware Supported
#define SFF_SEP         0x00000800      // Fast System Call
#define SFF_MTRR        0x00001000      // Memory Type Range Registers
#define SFF_PGE         0x00002000      // Page Global Enable
#define SFF_MCA         0x00004000      // Machine Check Architecture
#define SFF_CMOV        0x00008000      // Conditional Move Instruction Supported
#define SFF_PAT         0x00010000      // Page Attribute Table
#define SFF_PSE36       0x00020000      // 36-bit Page Size Extension
#define SFF_PSN         0x00040000      // Processor serial number is present and enabled (    AMD)
#define SFF_CLFSH       0x00080000      // CLFLUSH Instruction supported
#define SFF_DS          0x00200000      // Debug Store (    AMD)
#define SFF_ACPI        0x00400000      // Thermal Monitor and Software Controlled Clock Facilities supported (    AMD)
#define SFF_MMX         0x00800000      // Intel Architecture MMX technology supported
#define SFF_FXSR        0x01000000      // Fast floating point save and restore
#define SFF_SSE         0x02000000      // Streaming SIMD Extensions supported
#define SFF_SSE2        0x04000000      // Streaming SIMD Extensions 2
#define SFF_SS          0x08000000      // Self-Snoop (    AMD)
#define SFF_HTT         0x10000000      // Hyper-Threading Technology (    AMD)
#define SFF_TM          0x20000000      // Thermal Monitor supported (    AMD)
#define SFF_SBF         0x80000000      // Signal Break on FERR (    AMD)

/*
      (  Intel)
*/
#define IFF_TM2         0x00000080      // Thermal Monitor 2
#define IFF_EST         0x00000100      // Enhanced SpeedStep Technology
#define IFF_CID         0x00000400      // Context ID

/*
      (  AMD)
*/
#define AFF_SCSR        0x00000800      // SYSCALL and SYSRET Instructions
#define AFF_NPP         0x00010000      // No-execute Page Protection (NX)
#define AFF_MMXEX       0x00040000      // MMX Instruction Extensions
#define AFF_LMC         0x20000000      // Long Mode Capable (LM)
#define AFF_3DNOWEX     0x40000000      // 3DNow! Instruction Extensions
#define AFF_3DNOW       0x80000000      // 3DNow! Instructions


/*
     CPU
*/
typedef enum {
    VENDOR_INTEL   = 0x756E6547, // G(47h)e(65h)n(6Eh)u(75h) - GenuineIntel
    VENDOR_AMD     = 0x68747541, // A(41h)u(75h)t(74h)h(68h) - AuthenticAMD
    VENDOR_UNKNOWN = 0
} CPU_VENDOR;


/*
      CCpuInfo
*/
class CCpuInfo {

    private:
        bool _CpuId_Supported;
        char _VendorId[13];    // id 
        CPU_VENDOR _CpuVendor; // 

        dword _StdFeatures;      //  
        dword _ExtAMDFeatures;   //   (AMD)
        dword _ExtIntelFeatures; //   (Intel)

    public:
        CCpuInfo(void);
        const char* GetVendorId(void);
        const dword GetStdFeatures(void);
        const dword GetExtIntelFeatures(void);
        const dword GetExtAMDFeatures(void);
};

#endif  // __CPUINFO
