/*
 *  Copyright (C) 2002-2004 Aleksey Sugonyaev & 3OS.Team ( http://www.3os.ru/ )
 *  Licensed under the terms and conditions of the GNU Lesser General Public License
 */
#ifndef __DESCRIPTORS

/*
-----------------------------------------------------------------------------
 == 3OS kernel class definition module ===
   , 
   ""
-----------------------------------------------------------------------------
*/

#define __DESCRIPTORS

#include "ctype.hpp"
#include "macro.h"

/*   */
enum e_SpcGranulate {
 k_1Bt  =  0x0,             //   = 1 
 k_4Kb  =  0x00800000       //   = 4 
};

/*   */
enum e_SpcBpp {
 k_D16  =  0x0,
 k_D32  =  0x00400000
};

/*    */
enum e_DPLevel
{
  k_KLvl = 0,        // ""
  k_DLvl = 1 << 5,   // ""
  k_SLvl = 2 << 5,   // ""
  k_ALvl = 3 << 5    // ""
};


/*   - "" */
enum e_SgmState
{
  k_Swap = 0,        //  
  k_RAM  = 0x80      //  
};

/*  " "  */
enum e_SpcType
{
  k_RODataUp  = 0, // -        
  k_RWDataUp  = 1, // -        
  k_RODataDn  = 2, // -        
  k_RWDataDn  = 3, // -        
  k_EOCode    = 4, // -     
  k_ERCode    = 5, // -   c  
  k_EOCodeCnf = 6, // -     
  k_ERCodeCnf = 7  // -   c  
};


/*  "  "  */
enum e_SysType
{
  k_FreeTSS   = 1,  //  TSS
  k_LDT       = 2,  //   
  k_KeepTSS   = 3,  //  TSS
  k_CallGate  = 4,  //  
  k_TaskGate  = 5,  //  
  k_IntrGate  = 6,  //  
  k_TrapGate  = 7   //  
};

#pragma pack(1)
/*    */
struct s_DVector
{
  dword dwVec_I;
  dword dwVec_II;
};

/*   */
struct s_DSpace
{
   word  wLimit;
   word  wBase_I;
   byte  bBase_II;
   byte  bAttr_I;
   byte  bAttr_II;
   byte  bBase_III;
};

#ifndef __CODE16
/*    */
struct s_DSysObj
{
  word  wOffset_I;
  word  wSelector;
  byte  bCount;
  byte  bAttr;
  word  wOffset_II;
};

struct s_DSpcLimit
{
  union
  {
    dword  dwLimit;
    struct
    {
      word  wLimit_I;
      byte  bLimit_II;
      byte  bEmpty;
    };
  };

  dword  Size_Get(void)       {return Limit_Get()*((bLimit_II&0x80)? 4096 : 1);}
  dword  Limit_Get(void)      {return dwLimit & 0xFFFFF;};
  word   LimitI_Get(void)     {return wLimit_I;};
  byte   LimitII_Get(void)    {return bLimit_II;};
  void   G_Set(void)          {bLimit_II |= 0x80;};
  void   G_Clear(void)        {bLimit_II &= 0x7F;};
  bool   IsAddr32(void)       {return (bool)((bLimit_II & 0x40) == 0x40);};
  void   Addr16(void)         {bLimit_II &= 0xBF;};
  void   Addr32(void)         {bLimit_II |= 0x40;};
  s_DSpcLimit(dword nSize, dword mask=0x00400000)
   : dwLimit(nSize| mask) {};
};

struct s_SpcType
{
  byte bValue;

  void  Access_Clear(void)       {bValue &= 0xFE;};
  bool  IsAccessed(void)         {return (bool) (bValue & 0x01);};
  s_SpcType(e_SpcType e_SpType) : bValue((byte)(e_SpType<<1)|0x10) {};
};

struct s_DSysType
{
  byte  bValue;

  void  Addr32(void)          {bValue |= 0x08;};
  void  Addr16(void)          {bValue &= 0xF7;};
  e_SysType Get(void)         {return (e_SysType)(bValue & 0x07);};
  s_DSysType(e_SysType e_Type) : bValue((byte)((e_Type & 0xEF)|0x08)) {};
  s_DSysType(byte value)       : bValue(value) {};
};

struct s_ExtType
{
  byte   bValue;

  bool IsSystem(void)           {return (bool)((bValue & 0x10) == 0x00);};
  s_ExtType(byte value)         : bValue(value) {};
  s_DSysType SysType_Get(void)   {return s_DSysType(bValue);};
  s_ExtType(s_SpcType s_Space)  : bValue(s_Space.bValue) {};
  s_ExtType(s_DSysType s_SysObj) : bValue(s_SysObj.bValue) {};
};

class c_Descriptor
{
  public:
    union
    {
      s_DSpace   s_Space;
      s_DSysObj  s_SysObj;
      s_DVector  s_Vector;
    };

  public:
     c_Descriptor(e_DPLevel e_Level, s_ExtType s_EType, e_SgmState e_State){
      s_Space.bAttr_I = ((byte) e_Level)|s_EType.bValue|((byte) e_State);
     };
    ~c_Descriptor() {};

    void  Attr_Set(e_DPLevel e_Level, s_ExtType s_EType, e_SgmState e_State){
      s_Space.bAttr_I = ((byte) e_Level)|s_EType.bValue|((byte) e_State);
     };
    void  Level_Set(e_DPLevel e_Level);
    void  EType_Set(s_ExtType s_EType);
    void  Addr16(void);
    void  Addr32(void);
    void  Present(void);
    void  UnPresent(void);
    bool  IsSgmRAM(void);

    dword     Selector_Get(void);
    e_DPLevel Level_Get(void) {return (e_DPLevel)((dword) s_Space.bAttr_I&0x60);};
    void*     operator new(size_t);
};

class c_DSpace : public c_Descriptor
{
  public:
    c_DSpace(void *vBase, s_DSpcLimit s_Lim, s_ExtType s_Type, e_DPLevel e_Lvl);
   ~c_DSpace() {};

   void  Base_Set(void *vBase);
   void  Limit_Set(s_DSpcLimit s_Limit);
   void  Type_Set(s_SpcType s_Type);
};

#pragma pack(0)

extern m_Expand_ROR(void *vAddr);
#pragma aux m_Expand_ROR=\
 "shr  eax,  16" \
 "xchg  ah,  al" \
 "ror  eax,  8"  \
parm  [eax];

#endif

#endif