/*
 *  Copyright (C) 2002-2004 Aleksey Sugonyaev & 3OS.Team ( http://www.3os.ru/ )
 *  Licensed under the terms and conditions of the GNU Lesser General Public License
 */
#ifndef __HARDWARE
/*
 === 3OS Kernel definition hardware cpecifications ===
      
  , 
*/
#define __HARDWARE

#define k_PageSize     4096    //    0x86

extern void m_CPU_Stop(void);
#pragma aux m_CPU_Stop = "HLT";

#define m_Interrupt(imode) m_interrupt_##imode##()

extern void m_interrupt_Off(void);
#pragma aux m_interrupt_Off = "CLI";

extern void m_interrupt_On(void);
#pragma aux m_interrupt_On = "STI";

extern void m_interrupt_AllOff(void);
#pragma aux m_interrupt_AllOff = \
"  CLI"           \
"  MOV AL, 0x80"  \
"  OUT 0x70, AL"  \
modify [al];

#ifndef __CODE16
 enum e_EFLAGS
 {
   k_CARRY_FLAG  = 0x00000001,
   k_PARITY_FLAG = 0x00000004,
   k_A_FALG      = 0x00000010,
   k_ZERO_FLAG   = 0x00000040,
   k_SIGN_FLAG   = 0x00000080,
   k_TRACE_FLAG  = 0x00000100,
   k_INT_FLAG    = 0x00000200,
   k_DIR_FLAG    = 0x00000400,
   k_OVER_FLAG   = 0x00000800,
   k_IOPL_01     = 0x00001000,
   k_IOPL_02     = 0x00002000,
   k_IOPL_03     = 0x00003000,
   k_NT_FLAG     = 0x00004000,
   k_RF_FLAG     = 0x00010000,
   k_VM_FLAG     = 0x00020000,
   k_AC_FLAG     = 0x00040000,
   k_VIF_FLAG    = 0x00080000,
   k_VIP_FLAG    = 0x00100000,
   k_ID_FLAG     = 0x00200000
 };

 extern m_LoadGDTR(void *vAddr);
 #pragma aux m_LoadGDTR = \
 "LGDT [eax]" \
 parm[eax];

 extern m_SaveGDTR(void *vAddr);
 #pragma aux m_SaveGDTR = \
 "SGDT [eax]" \
 parm[eax];

 extern byte inp(word addr);
 #pragma aux inp=\
 "in al, dx"  \
 parm [dx] \
 modify[dx] \
 value[al];

 extern void outp(byte value, word addr);
 #pragma aux outp=\
 "out dx, al"  \
 parm [al][dx] \
 modify[dx];

 extern void m_JmpTo(void *vGoto);
 #pragma aux m_JmpTo = \
 "jmp edx"\
 parm [edx];

 struct s_Jmp_Far
 {
  byte   opCode;
  dword  offset;
  word   selector;
 };

 extern void m_Jmp_EDX(void *addr_jmp);
 #pragma aux m_Jmp_EDX = "jmp EDX" parm [EDX];

 extern void m_mask_CR4(dword Mask);
 #pragma aux m_mask_CR4 = \
 "mov eax, CR4"\
 "or  eax, ebx"\
 "mov CR4, eax"\
 parm [ebx];


/*===========     CPU ==============*/
#define SETSEG(name,value) SETSEG_##name##(value);

extern void SETESP(dword value);
#pragma aux SETESP = \
"mov esp,eax" \
parm [eax];

extern void SETSEG_SS(dword seg_value);
#pragma aux SETSEG_SS = \
"mov ss,ax" \
parm [ax];

extern void SETSEG_GS(dword seg_value);
#pragma aux SETSEG_GS = \
"mov gs,ax" \
parm [ax];

extern void SETSEG_FS(dword seg_value);
#pragma aux SETSEG_FS = \
"mov fs,ax" \
parm [ax];

extern void SETSEG_DS(dword seg_value);
#pragma aux SETSEG_DS = \
"mov ds,ax" \
parm [ax];

extern void SETSEG_ES(dword seg_value);
#pragma aux SETSEG_ES = \
"mov es,ax" \
parm [ax];
/* <<<<<<<<<<<<<<<<<<   SEG EOF  >>>>>>>>>>>>>>>>>>>>>>>> */

extern void cpuid(void far *p);
#pragma aux cpuid = \
"push gs"\
"mov gs, di"\
"db 0x0f, 0xa2"\
"mov es:[esi+0], ebx"\
"mov es:[esi+4], ecx"\
"mov es:[esi+8], edx"\
"mov es:[esi+12], eax"\
"pop gs"\
modify [ebx edx ecx]\
parm [di esi];

#endif

#endif