/*
 *  Copyright (C) 2002-2004 Aleksey Sugonyaev & 3OS.Team ( http://www.3os.ru/ )
 *  Licensed under the terms and conditions of the GNU Lesser General Public License
 */
//
// 3OS Kernel
// heap.cpp
// last update: 29.09.2003
// v.2.0

//#include <stdio.h>
#include "heap.hpp"

#ifdef HEAP_META

#define BLOCK_USE (1<<31)

#define bi_le(ptr) list_entry(ptr,TBlockInfo,le)

struct TBlockInfo{
  list_head le;
  dword addr;
  dword size;
};

static TBlockInfo* FindBlock(const list_head *list,void *addr){
  dword ap=(dword)addr;
  list_head *le;
  list_for_each(le,list){
    TBlockInfo *p=bi_le(le);
    if(ap>=p->addr && ap<(p->addr+(p->size&~BLOCK_USE)))
      return p;
  };
  return 0;
};

void THeap::Cleanup(void){
  base=0;
  mode=k_DWordAlign;
  old_mode=k_DWordAlign;
  heap_addr=0;
  heap_size=0;
  meta=0;
  meta_size=0;
  use_list.Init();
  free_list.Init();
};


void THeap::Init(void* aaddr,dword asize,int per){
  dword ams=((asize*per)/100)&(~3);
  dword ahs=asize-ams;
  Init(aaddr,ahs,((char*)aaddr)+ahs,ams);
};

void THeap::Init(void* aaddr,dword asize,void* ameta,dword ameta_size){
  Cleanup();
  heap_addr=aaddr;
  heap_size=asize;
  meta=ameta;
  meta_size=ameta_size;
  use_list.Init();
  free_list.Init();
  TBlockInfo *p=(TBlockInfo*)meta;
  int cnt=meta_size/sizeof(TBlockInfo);
  for(int i=0;i<cnt;i++){
    p[i].le.Init();
    list_add_tail(&p[i].le,&free_list);
  };
  p=bi_le(free_list.next);
//  printf("      p=%1X p->le=%1X\n",(int)p,(int)&p->le);
  list_del(free_list.next);
  p->addr=(dword)heap_addr;
  p->size=heap_size;
  list_add_tail(&p->le,&use_list);
//  printf("      p=%1X p->le=%1X\n",(int)p,(int)&p->le);
  base=(dword)heap_addr;
};

void* THeap::Alloc(dword sz){
  if(!sz)
    return 0;
  list_head *le;
  TBlockInfo *p;
  list_for_each(le,&use_list){
    p=bi_le(le);
    if((p->size&BLOCK_USE)!=0 || p->size<sz)
      continue;
    dword offs=p->addr-base;
    dword as=offs/mode;
    dword ds=offs%mode;
    if(ds)
      as++;
    dword roffs=as*mode;
    ds=roffs-offs;
//    printf("offs=%1X  ds=%1X\n",offs,ds);
    if((p->size-ds)<sz)
      continue;
    TBlockInfo *pb=0, *nb=0;
    if(ds){
      if(list_empty(&free_list))
        return 0;
      pb=bi_le(free_list.next);
      list_del(free_list.next);
    };
    dword nb_size=p->size-ds-sz;
//    printf("nb_size=%1X\n",nb_size);
    if(nb_size){
      if(list_empty(&free_list)){
//        printf("List empty 1\n");
        if(pb)
          list_add(&pb->le,&free_list);
          return 0;
      };
      nb=bi_le(free_list.next);
      list_del(free_list.next);
//      printf("nb=%1X  nb->le=%1X\n",(int)nb,(int)&nb->le);
    };
    if(pb){
      pb->addr=p->addr;
      pb->size=ds;
      p->addr+=ds;
      p->size-=ds;
      list_add_tail(&pb->le,&p->le);
    };
    if(nb){
      nb->addr=p->addr+sz;
      nb->size=nb_size;
      p->size-=nb_size;
      list_add(&nb->le,&p->le);
    };
    p->size|=BLOCK_USE;
    return (void*)p->addr;
  };
  return 0;
};

void* THeap::ReAlloc(void* P,dword sz){
  if(!P)
    return Alloc(sz);
  if(!sz){
    Free(P);
    return 0;
  };
  TBlockInfo *p=FindBlock(&use_list,P);
  if(!p)
    return 0;
  dword rsz=p->size&(~BLOCK_USE);
  TBlockInfo *nb=0;
  if(sz==rsz)
    return P;
  if(sz<rsz){
    if(list_empty(&free_list))
      return 0;
    nb=bi_le(free_list.next);
    list_del(free_list.next);
    nb->addr=p->addr+sz;
    nb->size=(rsz-sz)|BLOCK_USE;
    p->size=sz|BLOCK_USE;
    list_add(&nb->le,&p->le);
    Free((void*)nb->addr);
    return P;
  };
  nb=bi_le(p->le.next);
  if(p->le.next!=&use_list  &&  (nb->size & BLOCK_USE)==0  &&  (rsz+nb->size)>=sz){
    nb->size=(rsz+nb->size)-sz;
    nb->addr=p->addr+sz;
    p->size=sz|BLOCK_USE;
    if(!nb->size)
      Free((void*)nb->addr);
    return P;
  };
  void *tmp=Alloc(sz);
  if(!tmp)
    return 0;
  dword *src=(dword*)P;
  dword *dest=(dword*)tmp;
  dword cnt=sz/4;
  while(cnt--)
    *dest++=*src++;
  Free(P);
  return tmp;
};

void THeap::Free(void *P){
  if(!P)
    return;
  TBlockInfo *p=FindBlock(&use_list,P);
  if(!p)
    return;
  p->size&=~BLOCK_USE;
  TBlockInfo *t=0;
  if(p->le.prev!=&use_list){
    t=bi_le(p->le.prev);
    if( (t->size & BLOCK_USE) == 0 ){
      p->addr=t->addr;
      p->size+=t->size;
      list_del(&t->le);
      list_add(&t->le,&free_list);
    };
  };
  if(p->le.next!=&use_list){
    t=bi_le(p->le.next);
    if( (t->size & BLOCK_USE) == 0 ){
      p->size+=t->size;
      list_del(&t->le);
      list_add(&t->le,&free_list);
    };
  };
};

dword THeap::BlockSize(void *P)const{
  TBlockInfo *p=FindBlock(&use_list,P);
  return (p && (p->size&BLOCK_USE)!=0)?p->size&(~BLOCK_USE):0;
};

void THeap::SetAlignMode(e_HeapMode Mode){
  if(Mode==k_PrevAlign)
    mode=old_mode;
  else{
    old_mode=mode;
    mode=Mode;
  };
};

#else // HEAP_META

#include "ctype.hpp"
#include "mem.hpp"

void THeapBlock::CatNext(void){
  if(IsUse() || !next || next->IsUse())
    return;
  size+=SzHB+next->size;
  next=next->next;
  if(next)
    next->prev=this;
};

void THeapBlock::CatPrev(void){
  if(IsUse() || !prev || prev->IsUse())
    return;
  prev->CatNext();
};

void THeapBlock::Release(void){
  if(IsUse()){
    UnUse();
    CatNext();
    CatPrev();
  };
};

int THeapBlock::Alloc(unsigned int sz){
  if(sz>size)
    return 0;
  if((size-sz)<=SzHB){
    Use();
    return 1;
  };
  char *p=(char*)this;
  p+=SzHB+sz;
  THeapBlock *t=(THeapBlock*)p;
  t->next=next;
  if(next)
    next->prev=t;
  t->prev=this;
  t->size=size-(SzHB+sz);
  next=t;
  size=sz;
  Use();
  return 1;

};

void HBExpand(THeapBlock *p,unsigned int sz){
  p->UnUse();
  p->CatNext();
  p->Alloc(sz);
};

THeapBlock* THeapBlock::ReAlloc(unsigned int sz){
  THeapBlock *hb=this;
  unsigned int rsize=Size();
  if(!sz){
    Release();
    return 0;
  };
  if(sz==rsize){
    Use();
    return this;
  };
  if(sz<rsize){
    HBExpand(this,sz);
    return this;
  };
  unsigned int dsz=rsize;
  if(next && !next->IsUse())
    dsz+=rsize+SzHB;
  if(dsz>=sz){
    HBExpand(this,sz);
    return this;
  };
  if(prev && !prev->IsUse())
    dsz+=prev->size+SzHB;
  if(dsz<sz)
    return 0;
  dsz=rsize;
  void *p=DataOfHB(this);
  hb=prev;
  UnUse();
  CatNext();
  CatPrev();
  memmove(DataOfHB(hb),p,dsz);
  hb->Alloc(sz);
  return hb;
};

void THeap::Init(void* aaddr,unsigned int asize){
THeapBlock register *tmp;
THeapBlock register *NILL =(THeapBlock*) 0x0;

  List=NILL;
  if(!aaddr && asize<=SzHB){
    aaddr = NILL;                 //    aaddr   = 0
    asize = 0;
  }

  addr=aaddr;
  tmp = (THeapBlock*) aaddr;
  size=asize;
  List=tmp;                       // (THeapBlock*)addr;
  if(tmp){
    tmp->size=asize-SzHB;         // size-SzHB;
    tmp->next=tmp->prev=NILL;
  };
};

void* THeap::Alloc(unsigned int sz){
  if(!sz || !addr || sz>=size)
    return 0;
  unsigned int dmem=(unsigned int)(~0);
  THeapBlock *p=List,
             *best=0;
  while(p){
    if(!p->IsUse() && p->size>=sz){
      unsigned int x=p->size-sz;
      if(x<dmem){
        dmem=x;
        best=p;
        if(!dmem)
          break;
      };
    };
    p=p->next;
  };
  if(best && best->Alloc(sz))
    return DataOfHB(best);
  return 0;
};

void* THeap::ReAlloc(void* P,unsigned int sz){
  if(!P)
    return Alloc(sz);
  if(!sz || sz>=size){
    Free(P);
    return 0;
  };
  if(!IsContains(P))
    return 0;
  THeapBlock *b=HBOfData(P);
  THeapBlock *rb=b->ReAlloc(sz);
  if(rb)
    return DataOfHB(rb);
  void *t=Alloc(sz);
  if(!t){
    Free(P);
    return 0;
  };
  memcpy(t,P,BlockSize(P));
  Free(P);
  return t;
};

void THeap::Free(void *P){
  if(IsContains(P))
    HBOfData(P)->Release();
};

unsigned int THeap::BlockSize(void *P)const{
  return IsContains(P)?HBOfData(P)->Size():0;
};

#endif // HEAP_META

