/*
 *  Copyright (C) 2002-2004 Aleksey Sugonyaev & 3OS.Team ( http://www.3os.ru/ )
 *  Licensed under the terms and conditions of the GNU Lesser General Public License
 */
//
// 3OS Kernel definition module Kernel Heap definition and constants
//
// last update: 29.09.2003
// v2.0

#ifndef __HEAP_H
#define __HEAP_H

#include "conf3os.h"

#ifdef HEAP_META

#include "list.h"
#include "ctype.hpp"

#ifdef __GNUC__
#define HEAP_CALL_TYPE
#else
#define HEAP_CALL_TYPE __cdecl
#endif

enum e_HeapMode{
  k_ByteAlign  = 1,   
  k_WordAlign  = 2,   
  k_DWordAlign = 4,   
  k_QWordAlign = 8,   
  k_ParaAlign  = 16,   
  k_PageAlign  = 4096,   
  k_PrevAlign  = -1
};

class THeap{
private:
  dword base;
  e_HeapMode mode, old_mode;
  void *heap_addr;
  dword heap_size;
  void *meta;
  dword meta_size;
  list_head use_list, free_list;
  void Cleanup(void);
public:
  THeap(){ Cleanup(); };
  void Init(void* aaddr,dword asize){ Init(aaddr,asize,10); };
  void Init(void* aaddr,dword asize,int per);
  void Init(void* aaddr,dword asize,void* ameta,dword ameta_size);
  void* Alloc(dword sz);
  void* ReAlloc(void* P,dword sz);
  void Free(void *P);
  int IsContains(void *P)const{ return ((char*)P)>=((char*)heap_addr)&&((char*)P)<(((char*)heap_addr)+heap_size); };
  dword BlockSize(void *P)const;
  dword HeapSize(void)const{ return heap_size; };
  void* HeapAddr(void)const{ return heap_addr; };
  dword MetaSize(void)const{ return meta_size; };
  void* MetaAddr(void)const{ return meta; };
  void SetAlignMode(e_HeapMode Mode);
  e_HeapMode AlignMode(void)const{ return mode; };
  dword Base(void)const{ return base; };
  void SetBase(dword abase){ base=abase; };
};

extern THeap *pc_Heap;


inline void HEAP_CALL_TYPE *operator new(unsigned int size){
  return pc_Heap->Alloc(size);
};

inline void  HEAP_CALL_TYPE operator delete(void *ptr){
  pc_Heap->Free(ptr);
};

inline void  HEAP_CALL_TYPE *operator new []( unsigned int size ){
  return pc_Heap->Alloc(size);
}

#else // HEAP_META

#define HBUse (1<<31)

/*  ࠡ THeap  ᯥ祭  ࠢ 뤥  */
enum e_HeapMode{
  k_ByteAlign  = 0,   // ࠢ  
  k_WordAlign  = 2,   // ࠢ  ᫮
  k_DWordAlign = 3,   // ࠢ  " ᫮" 4 
  k_QWordAlign = 4,   // ࠢ   ࠣ (8 )
  k_ParaAlign  = 5,   // ࠢ  ࠣ (16 )
  k_PageAlign  = 6,   // ࠢ  ࠭ (4096 )
  k_PrevAlign  = 7    //   ।饬 ࠢ
};

struct THeapBlock{
  THeapBlock *prev, *next;
  unsigned int size;
  int IsUse(void)const{ return (size&HBUse)!=0; };
  void Use(void){ size|=HBUse; };
  void UnUse(void){ size&=~HBUse; };
  unsigned int Size(void)const{ return size&~HBUse; };
  void CatNext(void);
  void CatPrev(void);
  void Release(void);
  int Alloc(unsigned int sz);
  THeapBlock* ReAlloc(unsigned int sz);
};

#define SzHB (sizeof(THeapBlock))

inline void* DataOfHB(THeapBlock *p){
  return (void*)(((char*)p)+SzHB);
};

inline THeapBlock* HBOfData(void *p){
  return (THeapBlock*)(((char*)p)-SzHB);
};

class THeap{
private:
  void *addr;
  THeapBlock *List;
  unsigned int size;
public:
  THeap():addr(0),List(0),size(0){};
  void Init(void* aaddr,unsigned int asize);
  void* Alloc(unsigned int sz);
  void* ReAlloc(void* P,unsigned int sz);
  void Free(void *P);
  int IsContains(void *P)const{ return ((char*)P)>=((char*)addr)&&((char*)P)<(((char*)addr)+size); };
  unsigned int BlockSize(void *P)const;
  unsigned int HeapSize(void)const{ return size; };
  void* HeapAddr(void)const{ return addr; };
  void  SetAlignMode(e_HeapMode Mode) {};
};

extern THeap *pc_Heap;


inline void __cdecl *operator new(unsigned int size){
  return pc_Heap->Alloc(size);
};

inline void __cdecl operator delete(void *ptr){
  pc_Heap->Free(ptr);
};

inline void __cdecl *operator new []( unsigned int size ){
  return pc_Heap->Alloc(size);
}

#endif // HEAP_META

#endif // __HEAP_H

