/*
 *  Copyright (C) 2002-2004 Aleksey Sugonyaev & 3OS.Team ( http://www.3os.ru/ )
 *  Licensed under the terms and conditions of the GNU Lesser General Public License
 */
/*
-----------------------------------------------------------------------------
 ==== 3OS kernel module ===
 T   ++ 
-----------------------------------------------------------------------------
*/

#include "kernel.hpp"
#include "mem.hpp"
#include "heap.hpp"
#include "spaces.hpp"
#include "stream.hpp"
#include "cpuinfo.hpp"
#include "hardware.hpp"
#include "lkm\\lkm.hpp"
#include "v8086\\v8086.hpp"
#include "ievents.hpp"

extern c_Kernel  *pc_Krnl3OS;
extern char LKM_example[200];
extern char KrnlStream[84];
extern char v86[41];
extern s_DLCL DLCLStorage[2];

void* _DClassLink(int nClass)
{
 return DLCLStorage[nClass].vObject;
}

/*
 ⮤ ⠡樨 , ந ⠭  ன ⭮-
 ᨬ  .
*/
void c_Kernel::Stabilize(void* vBinPack)
{
   pc_Out = new c_Out(vBaseAddr);
   if (((s_BinPack*) vBinPack)->IsOK()) pc_Out->printf("\n\r\n\r startup.bin %d ",vBinPack);
   pc_Out->printf("PM > \001\x47 3OS Kernel \001\x2F allocated %d \n\r", (dword)GlobalAddr(0));
   pc_GDT = new c_GDT();
   pc_Out->printf("GDT located %d \n\r", pc_GDT->AddrGet());
   pc_GDT->ReLocate();
   pc_Out->printf("GDT relocated %d \n\r", pc_GDT->AddrGet());
 
  // ==== ࠭⢮   (ਯ ࠭ ⥭) ===
   pc_SpcCode = (c_CodeSpace*) pc_GDT->Dsc_Get(k_GDKrnlUpCode/8);

   pc_PIC = new c_PIC(0x20);

   pc_IDT = new c_IDT();
   pc_IDT->ISystem_Get(6)->TrapGate_Set(k_ALvl, pc_SpcCode, &_UD);
   pc_IDT->ISystem_Get(8)->TrapGate_Set(k_ALvl, pc_SpcCode, &_DF);
   pc_IDT->ISystem_Get(11)->TrapGate_Set(k_ALvl, pc_SpcCode, &_NP);
   pc_IDT->ISystem_Get(12)->TrapGate_Set(k_ALvl, pc_SpcCode, &_SS);
   pc_IDT->ISystem_Get(13)->TrapGate_Set(k_ALvl, pc_SpcCode, &_GP);
   pc_IDT->ISystem_Get(14)->TrapGate_Set(k_ALvl, pc_SpcCode, &_PF);


   pc_IDT->ISystem_Get(0x20+0x06)->TrapGate_Set(k_ALvl, pc_SpcCode, &_Interrupt_0E);
   pc_IDT->ISystem_Get(0x20)->TrapGate_Set(k_ALvl, pc_SpcCode, &_Interrupt_20);


   pc_IDT->ISystem_Get(0x10)->TrapGate_Set(k_ALvl, pc_SpcCode, &_Interrupt_10);
   pc_IDT->ISystem_Get(0x13)->TrapGate_Set(k_ALvl, pc_SpcCode, &_Interrupt_13);

#ifdef TEST_EXCEPTION
   _asm
   {
    mov ax, 0xfff
    mov ds, ax
    mov dword ptr ds:[eax], eax
   }
#endif

#ifdef CPU_DETECT
 CCpuInfo *CpuInfo = new CCpuInfo();

 pc_Out->puts("CPU Vendor Id: ");
 pc_Out->puts(CpuInfo->GetVendorId());
 if(CpuInfo->GetStdFeatures() & SFF_FPU)  pc_Out->puts("\n\rCPU Features: Built-in FPU");
 if(CpuInfo->GetStdFeatures() & SFF_MMX)  pc_Out->puts(", MMX");
 if(CpuInfo->GetStdFeatures() & SFF_SSE)  pc_Out->puts(", SSE");
 if(CpuInfo->GetStdFeatures() & SFF_SSE2) pc_Out->puts(", SSE2");
 if(CpuInfo->GetExtAMDFeatures() & AFF_3DNOW) pc_Out->puts(", 3DNOW!");
 if(CpuInfo->GetExtAMDFeatures() & AFF_3DNOWEX) pc_Out->puts(", Ext. 3DNOW!");
 pc_Out->puts("\n\r");
#endif

#ifdef SPACE_TEST
 pc_Out->puts("Test space construction\n\r");

 c_DataSpace *pc_TestSpace = new c_DataSpace(80*25*2, k_KLvl, (void*) 0xb8000);
 _asm{
  push es
   mov  ax, 0x40
   mov  es, ax
   mov  edi, 960
   mov  eax, 0x05440544
   mov  ecx, 160
   rep stosd
  pop  es
 }
#endif

#ifdef STREAM_LOAD
 pc_Out->puts("Threads stream ");

 if((*(dword*) KrnlStream) == 'TRDS')
  pc_Out->puts("\001\x2F found!\n\r");
 else
  pc_Out->puts("\001\x47 not found!\n\r");

 c_ThreadStream *pc_Stream = new c_ThreadStream((s_Stream*) &KrnlStream[4]);
 pc_Out->printf("Stream size %d, code %d\r\n",pc_Stream->Size(),pc_Stream->CntCodeSec());
#endif

#ifdef LKM_TEST
 c_LKM *pc_LKM = (c_LKM*) &LKM_example;
 if(pc_LKM->IsLKM()){
  pc_Out->puts("LKM found!");
  pc_LKM->Start(_DClassLink);
 }
#endif

#ifdef V86_TEST
/*
 ४祭   V8086
*/
 c_ThreadV86 *pc_Th86 = new c_ThreadV86();
 pc_Th86->IOEnable();
 pc_Th86->CopyFrom((void*)0xF000, &v86, 100);
 pc_Th86->SetESP(0xA000);
 pc_Th86->Switch();
#endif


#ifdef SWITCH_TEST
 SwitchTest(pc_Stream);
#endif

m1:
 m_interrupt_On();
 goto m1;
}

void c_Kernel::SwitchTest(c_ThreadStream *pc_Stream)
{
 c_Thread *pc_Th, *pc_ShareTh;
 pc_ThList = new c_ThList();

 /* ⠢  ।  ᮧ । N1 */
 pc_Th = pc_ThList->Add(new c_Item(new c_Thread(k_High, k_KLvl)))->Item;
 
 /* 㧨  ⮪ । */
 pc_Th->LoadFromStream(pc_Stream);
 pc_Th->SetESI(0x0); //(0xB8000);

 for(int i=4; i<180; i+=4){
   /* ᮧ । Ni */
   pc_ShareTh = new c_Thread(k_High, k_KLvl);
   /* ਬ */
   pc_ShareTh->Shared(pc_Th);
   pc_ShareTh->SetESI(i); 
   /* ⠢  । ७ । N2 */
   pc_ThList->Add(new c_Item(pc_ShareTh));
 }

 pc_ThList->Imdtly_Get()->Switch();
}

void c_Kernel::ICorruption(void){
// pc_Out->printf("Interrupt corruption vector!");
 pc_Out->gotoxy(1,20);
 pc_Out->printf("List = %d threads, Imdtly=%d \001\x47timer:\001\x1F %d", pc_ThList->Count(), pc_ThList->Imdtly_Get(), tcount++);
// pc_Out->puts("fsdfadf");
 pc_PIC->ResetIRS();
}

void c_Kernel::VM_Int10(dword dwEIP)
{
 pc_Out->gotoxy(1,20);
 pc_Out->printf("V8086 int 0x10 EIP = %d", dwEIP);
 while(1);
}

