/*
 *  Copyright (C) 2002-2004 Aleksey Sugonyaev & 3OS.Team ( http://www.3os.ru/ )
 *  Licensed under the terms and conditions of the GNU Lesser General Public License
 */
#ifndef __SPACES
/*
 === 3OS Kernel definition module class spaces ===
     3
*/
#define __SPACES
#include "descript.hpp"
#include "hardware.hpp"
#include "heap.hpp"

extern THeap *pc_Heap;

#pragma pack(1);

class c_Space : public c_DSpace{
 public:
   c_Space(void *vMem, dword nSize, e_DPLevel e_Lvl, e_SpcType e_SpType);
   c_Space(dword nSize, e_DPLevel e_Lvl, e_SpcType e_SpType);
   ~c_Space() {};

   void ReSize(dword nSize)   {};  //  
   void ReLocate(void *vAddr) {};  //   
};

class c_DataSpace : public c_Space{
 public:
  c_DataSpace(dword nSize, e_DPLevel e_Lvl)
   : c_Space(nSize, e_Lvl, k_RWDataUp) {};
  c_DataSpace(dword nSize, e_DPLevel e_Lvl, void *vMem)
   : c_Space(vMem, nSize, e_Lvl, k_RWDataUp) {};

  ~c_DataSpace() {};
};

class c_CodeSpace : public c_Space{
 public:
  c_CodeSpace(dword nSize, e_DPLevel e_Lvl)
   : c_Space(nSize, e_Lvl, k_ERCode) {};
  c_CodeSpace(dword nSize, e_DPLevel e_Lvl, void *vMem)
   : c_Space(vMem, nSize, e_Lvl, k_ERCode) {};

  ~c_CodeSpace() {};
};

class c_StackSpace : public c_Space{
 public:
  c_StackSpace(dword nSize, e_DPLevel e_Lvl)
   : c_Space(nSize, e_Lvl, k_RWDataUp) {};
  c_StackSpace(dword nSize, e_DPLevel e_Lvl, void *vMem)
   : c_Space(vMem, nSize, e_Lvl, k_RWDataUp) {};

  ~c_StackSpace() {};
};

struct s_PStack
{
 dword  dwESP;
 word   wSS;
 word   Reserv;
};

struct s_STDTSS
{
 word   wBackLink;
 word   wRes0;
 s_PStack       s_KrnlStack;
 s_PStack       s_DrvStack;
 s_PStack       s_SrvStack;
 dword  dwCR3;
 dword  dwEIP;
 dword  dwEFLAGS;
 dword  dwEAX;
 dword  dwECX;
 dword  dwEDX;
 dword  dwEBX;
 dword  dwESP;
 dword  dwEBP;
 dword  dwESI;
 dword  dwEDI;
 word   wES;
 word   wRes1;
 word   wCS;
 word   wRes2;
 word   wSS;
 word   wRes3;
 word   wDS;
 word   wRes4;
 word   wFS;
 word   wRes5;
 word   wGS;
 word   wRes6;
 word   wLDT;
 dword  dwRes7;
 word   wOffsBitMapIO;

 void TrapBit_Set(void) {dwRes7 |= 0x00010000;};
 void TrapBit_Clr(void) {dwRes7 &= 0xfffeffff;};
 void FlagVM_Set(void)  {dwEFLAGS |= (k_VM_FLAG|k_IOPL_03);};
 void BitMap_Set(dword dwCntx)  {wOffsBitMapIO = dwCntx-1-8*1024;};
};

struct s_386TSS : s_STDTSS
{
  char  aMapIO[8*1024];
  char  bFF;
};

struct s_PentiumTSS : s_STDTSS
{
  char  aMapINT[32];
  char  aMapIO[8*1024];
  char  bFF;
};

typedef void (SWCALL)(void);

struct s_JmpFar{
 byte  bOpcode;
 dword dwOffset;
 word  wSelector;
 byte  bRetOp;
};

const word k_CtxCommon  = sizeof(s_STDTSS);
const word k_Ctx386     = sizeof(s_386TSS);
const word k_CtxPentium = sizeof(s_PentiumTSS);

class c_Context{
 public:
  word      wCtx;
  s_STDTSS  *ps_TSS;
  c_DSpace  *pc_DSpace;

 protected:
  s_JmpFar  s_Switch;

 public:
  c_Context(e_DPLevel e_Lvl, dword dwContext);
  ~c_Context() {};

  void Clear(void);
  void Switch(void)            {((SWCALL*)&s_Switch)();};
  void SetESI(dword val)       {ps_TSS->dwESI = val;};
  void SetESP(dword val)       {ps_TSS->dwESP = val;};

  void SetEIP(dword dwStart)    {ps_TSS->dwEIP = dwStart;};
  void SetDS(word Seg)         {ps_TSS->wDS = Seg;};
  void SetES(word Seg)         {ps_TSS->wES = Seg;};
  void SetCS(word Seg)         {ps_TSS->wCS = Seg;};
  void VM_Set()                {ps_TSS->FlagVM_Set();};
};

#pragma pack(0);

#endif