/*
 *  Copyright (C) 2002-2004 Aleksey Sugonyaev & 3OS.Team ( http://www.3os.ru/ )
 *  Licensed under the terms and conditions of the GNU Lesser General Public License
 */
// 3OS Vesa module header
// last update 10.12.03

#ifndef _VESAINFO_H_
#define _VESAINFO_H_

//#include "ctype.hpp"

#define VESA_OK			0
#define VESA_ERROR		1

//Vesa 1.0 & 1.1
// 
#define MODE640X400X256		0x0100
#define MODE640X480X256		0x0101
#define MODE800X600X16		0x0102
#define MODE800X600X256		0x0103
#define MODE1024X768X16		0x0104
#define MODE1024X768X256	0x0105
#define MODE1280X1024X16	0x0106
#define MODE1280X1024X256	0x0107
// 
#define MODE80X60X16		0x0108
#define MODE132X25X16		0x0109
#define MODE132X43X16		0x010A
#define MODE132X50X16		0x010B
#define MODE132X60X16		0x010C
//Vesa 1.2
// 
#define MODE320X200X32K		0x010D
#define MODE320X200X64K		0x010E
#define MODE320X200X16M		0x010F
#define MODE640X480X32K		0x0110
#define MODE640X480X64K		0x0111
#define MODE640X480X16M		0x0112
#define MODE800X600X32K		0x0113
#define MODE800X600X64K		0x0114
#define MODE800X600X16M		0x0115
#define MODE1024X768X32K	0x0116
#define MODE1024X768X64K	0x0117
#define MODE1024X768X16M	0x0118
#define MODE1280X1024X32K	0x0119
#define MODE1280X1024X64K	0x011A
#define MODE1280X1024X16M	0x011B

typedef unsigned short us;
typedef unsigned int   ui;
typedef unsigned char  uc;
typedef unsigned long  ul;

#define FP_OFF(__p) ((unsigned)(__p))
#define FP_SEG(__p) ((unsigned)((unsigned long)(void __far*)(__p) >> 16)) 

typedef struct SVBE_InfoBlock {
	uc VbeSignature[4];           // VBE Signature
	us VbeVersion;                // VBE Version
	ul OemStringPtr;              // Pointer to OEM String
	ul Capabilities;              // Capabilities of graphics cont.
	ul VideoModePtr;              // Pointer to Video Mode List
	// Added for VBE 1.2+
	us TotalMemory;               // Number of 64kb memory blocks
	// Added for VBE 2.0+
	us OemSoftwareRev;            // VBE implementation Software revision
	ul OemVendorNamePtr;          // Pointer to Vendor Name String
	ul OemProductNamePtr;         // Pointer to Product Name String
	ul OemProductRevPtr;          // Pointer to Product Revision String
	us AccelVbeVersion;           // VBE/AF Version
	ul AccelVideoModePtr;         // Pointer to Acclelerated Mode List
	uc Reserved20[216];           // Reserved for VBE implementation
	// scratch area
	uc OemData[256];              // Data Area for OEM Strings
} VBE_InfoBlock;

typedef struct SVBE_ModeInfoBlock {
	us ModeAttributes;            // mode attributes
	uc WinAAttributes;            // window A attributes
	uc WinBAttributes;            // window B attributes
	us WinGranularity;            // window granularity
	us WinSize;                   // window size
	us WinASegment;               // window A start segment
	us WinBSegment;               // window B start segment
	ul WinFuncPtr;                // pointer to window function
	us BytesPerScanLine;          // bytes per scan line
	// Mandatory information for VBE 1.2 and above
	us XResolution;               // horizontal resolution in pixels or chars
	us YResolution;               // vertical resolution in pixels or chars
	uc XCharSize;                 // character cell width in pixels
	uc YCharSize;                 // character cell height in pixels
	uc NumberOfPlanes;            // number of memory planes
	uc BitsPerPixel;              // bits per pixel
	uc NumberOfBanks;             // number of banks
	uc MemoryModel;               // memory model type
	uc BankSize;                  // bank size in KB
	uc NumberOfImagePages;        // number of images
	uc Reserved12;                // reserved for page function
	// Direct Color fields (required
	// for direct/6 and YUV/7 memory models)
	uc RedMaskSize;               // size of direct color red mask in bits
	uc RedFieldPosition;          // bit position of lsb of red mask
	uc GreenMaskSize;             // size of direct color green mask in bits
	uc GreenFieldPosition;        // bit position of lsb of green mask
	uc BlueMaskSize;              // size of direct color blue mask in bits
	uc BlueFieldPosition;         // bit position of lsb of blue mask
	uc RsvdMaskSize;              // size of direct color reserved mask in bits
	uc RsvdFieldPosition;         // bit position of lsb of reserved mask
	uc DirectColorModeInfo;       // direct color mode attributes
	// Mandatory information for VBE 2.0 and above
	ul PhysBasePtr;               // physical address for flat frame buffer
	ul OffScreenMemOffset;        // pointer to start of off screen memory
	us OffScreenMemSize;          // amount of off screen memory in 1k units
	uc Reserved20[206];           // remainder of ModeInfoBlock
} VBE_ModeInfoBlock;

class CVesaInfo {
public:
	CVesaInfo();
	~CVesaInfo();

	int					GetVESAInfo();
	int					GetVESAModeInfo(us mode);
	uc					isVesaSignatureOK();
protected:
	VBE_InfoBlock		info;
	VBE_ModeInfoBlock	modeinfo;
};

#endif//_VESAINFO_H_